/*============================================================================
  Summary:  Demonstrates In-Memory OLTP
------------------------------------------------------------------------------
  Written by Klaus Aschenbrenner, www.SQLpassion.at

  For more information about SQL Server, check out my website
    http://www.SQLpassion.at
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

USE master
GO

-- Create new database
CREATE DATABASE InMemoryOLTP
GO

--Add MEMORY_OPTIMIZED_DATA filegroup to the database.
ALTER DATABASE InMemoryOLTP
ADD FILEGROUP InMemoryOLTPFileGroup CONTAINS MEMORY_OPTIMIZED_DATA
GO

USE InMemoryOLTP
GO

-- Add a new file to the previous created file group
ALTER DATABASE InMemoryOLTP ADD FILE
(
	NAME = N'InMemoryOLTPContainer', 
	FILENAME = N'C:\Program Files\Microsoft SQL Server\MSSQL12.SQL2014\MSSQL\DATA\InMemoryOLTPContainer'
)
TO FILEGROUP [InMemoryOLTPFileGroup]
GO

-- Let's create a new Memory Optimized Table
CREATE TABLE TestTable
(
	Col1 INT NOT NULL,
	Col2 VARCHAR(100) NOT NULL,
	Col3 VARCHAR(100) NOT NULL
	CONSTRAINT chk_PrimaryKey PRIMARY KEY NONCLUSTERED HASH (Col1) WITH (BUCKET_COUNT = 1024) 
) WITH (MEMORY_OPTIMIZED = ON)
GO

-- Let's insert a simple record into the new table
INSERT INTO TestTable (Col1, Col2, Col3) VALUES (2, 'Klaus', 'Aschenbrenner')
GO

-- Make an insert in an explicit transaction
BEGIN TRANSACTION

INSERT INTO TestTable (Col1, Col2, Col3) VALUES (4, 'Philip', 'Aschenbrenner')

-- No IX, X locks anymore!
SELECT * FROM sys.dm_tran_locks
WHERE request_session_id = @@SPID

COMMIT
GO

-- Retrieve the data from the table, SQL Server uses an "Index Scan (NonClusteredHash)" operator in the Execution Plan
SELECT * FROM TestTable
GO

-- Simple UPDATE statement, SQL Server uses an "Index Seek (NonClusteredHash)" operator in the Execution Plan
UPDATE TestTable SET Col2 = 'Test' WHERE Col1 = 1
GO

-- Create a natively compiled Stored Procedure
CREATE PROCEDURE InMemoryOLTPProcedure
(
   @Param INT
)
WITH NATIVE_COMPILATION, SCHEMABINDING, EXECUTE AS OWNER
AS
BEGIN
   ATOMIC WITH
   (
      TRANSACTION ISOLATION LEVEL = SNAPSHOT, LANGUAGE = 'us_english'
   )

   INSERT INTO dbo.TestTable (Col1, Col2, Col3) VALUES (@param, 'Klaus', 'Aschenbrenner')

   SELECT Col1, Col2, Col3 FROM dbo.TestTable
END
GO

EXEC InMemoryOLTPProcedure 6
GO

-- Clean up
USE master
GO

DROP DATABASE InMemoryOLTP
GO